@@ -2,10 +2,11 @@ import json
 import os
 import shutil
 import sys
+import warnings
 from collections import OrderedDict  # can be removed from python 3.7
 from distutils.version import LooseVersion
 from pathlib import Path
-from typing import Any, Dict, List, MutableMapping, Optional
+from typing import Any, Dict, List, MutableMapping, Optional, Set, Union, cast
 
 import requests
 import typer
@@ -32,6 +33,8 @@ from controller.project import ANGULAR, NO_FRONTEND, Project
 from controller.templating import Templating
 from controller.utilities import configuration, services, system
 
+warnings.simplefilter("always", DeprecationWarning)
+
 DataFileStub = Dict[str, List[str]]
 
 ROOT_UID = 0
@@ -49,21 +52,22 @@ class Configuration:
 
     action: Optional[str] = None
 
-    production = False
-    testing = False
-    privileged = False
+    production: bool = False
+    testing: bool = False
+    privileged: bool = False
     project: str = ""
-    frontend = None
+    frontend: Optional[str] = None
     hostname: str = ""
     stack: str = ""
-    load_backend = False
-    load_frontend = False
-    load_commons = False
+    load_backend: bool = False
+    load_frontend: bool = False
+    load_commons: bool = False
 
-    version: Optional[str] = None
-    rapydo_version: Optional[str] = None
+    version: str = ""
+    rapydo_version: str = ""
     project_title: Optional[str] = None
     project_description: Optional[str] = None
+    project_keywords: Optional[str] = None
 
     initialize: bool = False
     update: bool = False
@@ -235,7 +239,7 @@ class CommandsData:
         active_services: List[str] = [],
         base_services: List[Any] = [],
         compose_config: List[Any] = [],
-        services_dict: Any = None,
+        services_dict: Dict[str, Any] = None,
     ):
         self.files = files
         self.base_files = base_files
@@ -244,7 +248,7 @@ class CommandsData:
         self.active_services = active_services or []
         self.base_services = base_services
         self.compose_config = compose_config
-        self.services_dict: Dict[str, Any] = services_dict or {}
+        self.services_dict = services_dict or {}
 
 
 class Application:
@@ -259,10 +263,9 @@ class Application:
     controller: Optional["Application"] = None
     project_scaffold = Project()
     data = CommandsData()
-    # This Any should be Repo, but GitPython is lacking typings
-    gits: MutableMapping[str, Any] = OrderedDict()
+    gits: MutableMapping[str, gitter.GitRepoType] = OrderedDict()
 
-    def __init__(self):
+    def __init__(self) -> None:
 
         Application.controller = self
 
@@ -273,14 +276,14 @@ class Application:
         self.enabled_services: List[str] = []
         self.base_services: List[Any] = []
         self.compose_config: List[Any] = []
-        self.services_dict = {}
+        self.services_dict: Dict[str, List[Any]] = {}
 
         load_commands()
 
         Application.load_projectrc()
 
     @staticmethod
-    def exit(message, *args, **kwargs):
+    def exit(message: str, *args: Union[str, Path], **kwargs: Union[str, Path]) -> None:
         log.critical(message, *args, **kwargs)
         sys.exit(1)
 
@@ -351,14 +354,11 @@ class Application:
         # Cannot be tested
         if self.current_uid == ROOT_UID:  # pragma: no cover
             self.current_uid = BASE_UID
-            self.current_os_user = "privileged"
             log.warning("Current user is 'root'")
         else:
-            self.current_os_user = system.get_username(self.current_uid)
-            log.debug(
-                "Current user: {} (UID: {})", self.current_os_user, self.current_uid
-            )
-            log.debug("Current group ID: {}", self.current_gid)
+            os_user = system.get_username(self.current_uid)
+            log.debug("Current UID: {} ({})", self.current_uid, os_user)
+            log.debug("Current GID: {}", self.current_gid)
 
         if Configuration.initialize:
             return None
@@ -392,7 +392,7 @@ class Application:
         return None
 
     @staticmethod
-    def load_projectrc():
+    def load_projectrc() -> None:
 
         projectrc_yaml = configuration.load_yaml_file(
             PROJECTRC, path=Path(), is_optional=True
@@ -405,7 +405,7 @@ class Application:
         Configuration.projectrc = projectrc_yaml
 
     @staticmethod
-    def check_installed_software():
+    def check_installed_software() -> None:
 
         log.debug(
             "python version: {}.{}.{}",
@@ -415,8 +415,9 @@ class Application:
         )
 
         # 17.05 added support for multi-stage builds
+        # https://docs.docker.com/compose/compose-file/compose-file-v3/#compose-and-docker-compatibility-matrix
         Packages.check_program(
-            "docker", min_version="17.05", min_recommended_version="19.03.1"
+            "docker", min_version="17.05", min_recommended_version="19.03.8"
         )
         Packages.check_program("git")
 
@@ -428,7 +429,7 @@ class Application:
         Packages.check_python_package("requests", min_version="2.6.1")
         Packages.check_python_package("pip", min_version="10.0.0")
 
-    def read_specs(self, read_extended=True):
+    def read_specs(self, read_extended: bool = True) -> None:
         """ Read project configuration """
 
         try:
@@ -448,7 +449,7 @@ class Application:
             self.extended_project_path = confs[2]
 
         except AttributeError as e:  # pragma: no cover
-            Application.exit(e)
+            Application.exit(str(e))
 
         Configuration.frontend = glom(
             Configuration.specs, "variables.env.FRONTEND_FRAMEWORK", default=NO_FRONTEND
@@ -460,18 +461,20 @@ class Application:
         Configuration.project_title = glom(
             Configuration.specs, "project.title", default="Unknown title"
         )
-        Configuration.version = glom(
-            Configuration.specs, "project.version", default=None
-        )
+        Configuration.version = glom(Configuration.specs, "project.version", default="")
         Configuration.rapydo_version = glom(
-            Configuration.specs, "project.rapydo", default=None
+            Configuration.specs, "project.rapydo", default=""
         )
 
         Configuration.project_description = glom(
             Configuration.specs, "project.description", default="Unknown description"
         )
 
-        if Configuration.rapydo_version is None:  # pragma: no cover
+        Configuration.project_keywords = glom(
+            Configuration.specs, "project.keywords", default=""
+        )
+
+        if not Configuration.rapydo_version:  # pragma: no cover
             Application.exit(
                 "RAPyDo version not found in your project_configuration file"
             )
@@ -479,7 +482,7 @@ class Application:
         Configuration.rapydo_version = str(Configuration.rapydo_version)
 
     @staticmethod
-    def preliminary_version_check():
+    def preliminary_version_check() -> None:
 
         specs = configuration.load_yaml_file(
             file=configuration.PROJECT_CONF_FILENAME,
@@ -488,19 +491,19 @@ class Application:
         )
 
         Application.verify_rapydo_version(
-            rapydo_version=glom(specs, "project.rapydo", default=None)
+            rapydo_version=glom(specs, "project.rapydo", default="")
         )
 
     @staticmethod
-    def verify_rapydo_version(rapydo_version=None):
+    def verify_rapydo_version(rapydo_version: str = "") -> bool:
         """
         Verify if the installed rapydo version matches the current project requirement
         """
 
-        if rapydo_version is None:
+        if not rapydo_version:
             rapydo_version = Configuration.rapydo_version
 
-        if rapydo_version is None:  # pragma: no cover
+        if not rapydo_version:  # pragma: no cover
             return True
 
         r = LooseVersion(rapydo_version)
@@ -519,10 +522,11 @@ class Application:
                 r, c, action
             )
 
-            Application.exit(msg)
+            log.critical(msg)
+            sys.exit(1)
 
     @staticmethod
-    def check_internet_connection():
+    def check_internet_connection() -> None:
         """ Check if connected to internet """
 
         try:
@@ -532,6 +536,7 @@ class Application:
         except requests.ConnectionError:  # pragma: no cover
             Application.exit("Internet connection is unavailable")
 
+    # from_path: Optional[Path]
     @staticmethod
     def working_clone(name, repo, from_path=None):
 
@@ -578,10 +583,14 @@ class Application:
         )
 
     @staticmethod
-    def git_submodules(from_path=None):
+    def git_submodules(from_path: Optional[Path] = None) -> None:
         """ Check and/or clone git projects """
 
-        repos = glom(Configuration.specs, "variables.submodules", default={}).copy()
+        repos: Dict[str, str] = glom(
+            Configuration.specs,
+            "variables.submodules",
+            default=cast(Dict[str, str], {}),
+        ).copy()
         Application.gits["main"] = gitter.get_repo(".")
 
         for name, repo in repos.items():
@@ -589,7 +598,7 @@ class Application:
                 name, repo, from_path=from_path
             )
 
-    def set_active_services(self):
+    def set_active_services(self) -> None:
         self.services_dict, self.active_services = services.find_active(
             self.compose_config
         )
@@ -600,7 +609,7 @@ class Application:
 
         self.create_datafile()
 
-    def read_composers(self):
+    def read_composers(self) -> None:
 
         # Find configuration that tells us which files have to be read
 
@@ -629,7 +638,9 @@ class Application:
 
         compose_files = OrderedDict()
 
-        confs = glom(Configuration.specs, "variables.composers", default={})
+        confs: Dict[str, Any] = glom(
+            Configuration.specs, "variables.composers", default={}
+        )
         for name, conf in confs.items():
             compose_files[name] = services.apply_variables(conf, myvars)
 
@@ -643,7 +654,7 @@ class Application:
         dc = Compose(files=self.files)
         self.compose_config = dc.config()
 
-    def create_projectrc(self):
+    def create_projectrc(self) -> None:
         templating = Templating()
         t = templating.get_template(
             "projectrc",
@@ -665,14 +676,14 @@ class Application:
         else:
             log.info("Created default {} file", PROJECTRC)
 
-    def make_env(self):
+    def make_env(self) -> None:
 
         try:
             COMPOSE_ENVIRONMENT_FILE.unlink()
         except FileNotFoundError:
             pass
 
-        env = glom(Configuration.specs, "variables.env", default={})
+        env: Dict[str, Any] = glom(Configuration.specs, "variables.env", default={})
 
         env["PROJECT_DOMAIN"] = Configuration.hostname
         env["COMPOSE_PROJECT_NAME"] = Configuration.project
@@ -698,6 +709,7 @@ class Application:
         env["CURRENT_GID"] = self.current_gid
         env["PROJECT_TITLE"] = Configuration.project_title
         env["PROJECT_DESCRIPTION"] = Configuration.project_description
+        env["PROJECT_KEYWORDS"] = Configuration.project_keywords
         env["DOCKER_PRIVILEGED_MODE"] = "true" if Configuration.privileged else "false"
 
         if Configuration.testing:
@@ -708,6 +720,8 @@ class Application:
         env["DOCKER_NETWORK_MODE"] = "bridge"
 
         services.check_rabbit_password(env.get("RABBITMQ_PASSWORD"))
+        services.check_redis_password(env.get("REDIS_PASSWORD"))
+        services.check_mongodb_password(env.get("MONGO_PASSWORD"))
 
         for e in env:
             env_value = os.environ.get(e)
@@ -717,15 +731,56 @@ class Application:
 
         env.update(Configuration.environment)
 
+        bool_envs = [
+            # This variable is for docker-compose and is expected to be true|false
+            "DOCKER_PRIVILEGED_MODE",
+            # This variable is for RabbitManagement and is expected to be true|false
+            "RABBITMQ_SSL_FAIL_IF_NO_PEER_CERT",
+            # These variables are for Neo4j and are expected to be true|false
+            "NEO4J_SSL_ENABLED",
+            "NEO4J_ALLOW_UPGRADE",
+            "NEO4J_RECOVERY_MODE",
+        ]
         with open(COMPOSE_ENVIRONMENT_FILE, "w+") as whandle:
             for key, value in sorted(env.items()):
 
-                # if isinstance(value, str):  # pragma: no
-                #     if value.lower() == 'true':
-                #         log.warning("{}={}, convert to 1?", key, value)
+                if (
+                    Configuration.production
+                    and key.endswith("_PASSWORD")
+                    and value
+                    and len(value) < 8
+                ):
+                    log.warning("{} is set with a short password", key)
 
-                #     if value.lower() == 'false':
-                #         log.warning("{}={}, convert to 0?", key, value)
+                # Deprecated since 1.0
+                # Backend and Frontend use different booleans due to Py vs Js
+                # 0/1 is a much more portable value to prevent true|True|"true"
+                # This fixes troubles in setting boolean values only used by Angular
+                # (expected true|false) or used by Pyton (expected True|False)
+                if key not in bool_envs:  # pragma: no cover
+                    if isinstance(value, str):
+                        if value.lower() == "true":
+                            warnings.warn(
+                                f"Deprecated value for {key}, convert {value} to 1",
+                                DeprecationWarning,
+                            )
+
+                        if value.lower() == "false":
+                            warnings.warn(
+                                f"Deprecated value for {key}, convert {value} to 0",
+                                DeprecationWarning,
+                            )
+                    elif isinstance(value, bool):
+                        if value:
+                            warnings.warn(
+                                f"Deprecated value for {key}, convert {value} to 1",
+                                DeprecationWarning,
+                            )
+                        else:
+                            warnings.warn(
+                                f"Deprecated value for {key}, convert {value} to 0",
+                                DeprecationWarning,
+                            )
 
                 if value is None:
                     value = ""
@@ -735,7 +790,7 @@ class Application:
                     value = f"'{value}'"
                 whandle.write(f"{key}={value}\n")
 
-    def create_datafile(self):
+    def create_datafile(self) -> None:
         try:
             DATAFILE.unlink()
         except FileNotFoundError:
@@ -776,7 +831,7 @@ class Application:
             return values
         return [x for x in values if x.startswith(incomplete)]
 
-    def get_available_interfaces(self):
+    def get_available_interfaces(self) -> List[str]:
         available_interfaces = list()
         if self.compose_config:
             for s in self.compose_config:
@@ -815,7 +870,7 @@ class Application:
             return values
         return [x for x in values if x.startswith(incomplete)]
 
-    def check_placeholders(self):
+    def check_placeholders(self) -> Set[str]:
 
         if len(self.active_services) == 0:  # pragma: no cover
             Application.exit(
@@ -832,7 +887,9 @@ and add the variable "ACTIVATE_DESIREDSERVICE: 1"
             service = self.services_dict.get(service_name)
 
             if service:
-                for key, value in service.get("environment", {}).items():
+                for key, value in (
+                    cast(Dict[str, Any], service).get("environment", {}).items()
+                ):
                     if PLACEHOLDER in str(value):
                         key = services.normalize_placeholder_variable(key)
                         missing.add(key)
@@ -866,24 +923,63 @@ and add the variable "ACTIVATE_DESIREDSERVICE: 1"
         if len(placeholders) > 0:
             Application.exit(
                 "The following variables are missing in your configuration:\n\n{}"
-                "\n\nYou can fix this error by updating the your .projectrc file\n",
+                "\n\nYou can fix this error by updating your .projectrc file\n",
                 "\n".join(placeholders),
             )
 
         return missing
 
     @staticmethod
-    def git_update(ignore_submodule):
+    def git_update(ignore_submodule: List[str]) -> None:
 
         for name, gitobj in Application.gits.items():
             if name in ignore_submodule:
                 log.debug("Skipping update on {}", name)
                 continue
+
+            if gitobj and not gitter.can_be_updated(name, gitobj):
+                Application.exit("Can't continue with updates")
+
+        controller_is_updated = False
+        for name, gitobj in Application.gits.items():
+            if name in ignore_submodule:
+                continue
+
+            if name == "do":
+                controller_is_updated = True
+
             if gitobj:
                 gitter.update(name, gitobj)
 
+        if controller_is_updated:
+            installation_path = Packages.get_installation_path("rapydo")
+            if installation_path:
+                do_dir = Path(Application.gits["do"].working_dir)
+                if do_dir.is_symlink():
+                    do_dir = do_dir.resolve()
+                    # This can be used starting from python 3.9
+                    # do_dir = do_dir.readlink()
+
+                if do_dir == installation_path:
+                    log.info(
+                        "Controller installed from {} and updated", installation_path
+                    )
+                else:
+                    log.warning(
+                        "Controller not updated because it is installed outside this "
+                        "project. Installation path is {}, the current folder is {}",
+                        installation_path,
+                        do_dir,
+                    )
+            # Can't be tested on GA since rapydo is alway installed from a folder
+            else:  # pragma: no cover
+                log.warning(
+                    "Controller is not installed in editable mode, "
+                    "rapydo is unable to update it"
+                )
+
     @staticmethod
-    def git_checks(ignore_submodule):
+    def git_checks(ignore_submodule: List[str]) -> None:
 
         for name, gitobj in Application.gits.items():
             if name in ignore_submodule:
