@@ -2,7 +2,7 @@ from importlib import import_module
 from pathlib import Path
 
 
-def load_commands():
+def load_commands() -> None:
 
     commands_folder = Path(__file__).resolve().parent
 
