@@ -1,4 +1,5 @@
 import sys
+from typing import List
 
 import docker
 import requests
@@ -8,25 +9,20 @@ from controller import log
 
 
 class Dock:
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
 
         self.client = docker.from_env()
-
-        if not self.is_daemon_alive():  # pragma: no cover
-            log.critical("Docker daemon not reachable")
-            sys.exit(1)
-
-    def is_daemon_alive(self):
-
         try:
-            return self.client.ping()
+            self.client.ping()
         # this is the case of docker daemon not started
         except requests.exceptions.ConnectionError:  # pragma: no cover
-            return False
-        # this is the case of docker daemon starting or not working properly
+            log.critical("Docker daemon not reachable")
+            sys.exit(1)
+        # this is the case of docker daemon still starting or not working properly
         except APIError:  # pragma: no cover
-            return False
+            log.critical("Docker daemon not reachable")
+            sys.exit(1)
 
     def image_info(self, tag):
         obj = self.client.images.list(name=tag)
@@ -35,10 +31,7 @@ class Dock:
         except IndexError:
             return {}
 
-    def image_attribute(self, tag, attribute="Created"):
-        return self.image_info(tag).get(attribute, None)
-
-    def images(self):
+    def images(self) -> List[str]:
         images = []
         for obj in self.client.images.list():
             tags = obj.attrs.get("RepoTags")
